package com.ruoyi.web.controller.dragForm;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.system.entity.CommonEntity;
import com.ruoyi.system.entity.TableInfo;
import com.ruoyi.system.service.ICommonService;
import com.ruoyi.system.service.IDataSourceService;
import io.lettuce.core.ScriptOutputType;
import io.swagger.annotations.ApiOperation;
import io.swagger.models.auth.In;
import org.apache.poi.ss.formula.functions.T;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.ss.usermodel.*;

import static com.ruoyi.common.core.domain.AjaxResult.success;

/**
 * 通用请求处理
 *
 * @author ruoyi
 */
@Anonymous
@RestController
@RequestMapping("/common")
public class CommonFileController {
    private static final Logger log = LoggerFactory.getLogger(CommonController.class);

    @Autowired
    private ServerConfig serverConfig;

    @Resource
    private IDataSourceService dataSourceService;

    @Resource
    private ICommonService commonService;

    private static final String FILE_DELIMETER = ",";

    /**
     * 通用下载请求
     *
     * @param fileName 文件名称
     * @param delete   是否删除
     */
    @GetMapping("/download")
    public void fileDownload(String fileName, Boolean delete, HttpServletResponse response, HttpServletRequest request) {
        try {
            if (!FileUtils.checkAllowDownload(fileName)) {
                throw new Exception(StringUtils.format("文件名称({})非法，不允许下载。 ", fileName));
            }
            String realFileName = System.currentTimeMillis() + fileName.substring(fileName.indexOf("_") + 1);
            String filePath = RuoYiConfig.getDownloadPath() + fileName;

            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, realFileName);
            FileUtils.writeBytes(filePath, response.getOutputStream());
            if (delete) {
                FileUtils.deleteFile(filePath);
            }
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    /**
     * 通用上传请求（单个）
     */
    @PostMapping("/upload")
    public AjaxResult uploadFile(MultipartFile file) throws Exception {
        try {
            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, file);
            String url = serverConfig.getUrl() + fileName;
            AjaxResult ajax = success();
            ajax.put("url", url);
            ajax.put("fileName", fileName);
            ajax.put("newFileName", FileUtils.getName(fileName));
            ajax.put("originalFilename", file.getOriginalFilename());
            return ajax;
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 通用上传请求（多个）
     */
    @PostMapping("/uploads")
    public AjaxResult uploadFiles(List<MultipartFile> files) throws Exception {
        try {
            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            List<String> urls = new ArrayList<String>();
            List<String> fileNames = new ArrayList<String>();
            List<String> newFileNames = new ArrayList<String>();
            List<String> originalFilenames = new ArrayList<String>();
            for (MultipartFile file : files) {
                // 上传并返回新文件名称
                String fileName = FileUploadUtils.upload(filePath, file);
                String url = serverConfig.getUrl() + fileName;
                urls.add(url);
                fileNames.add(fileName);
                newFileNames.add(FileUtils.getName(fileName));
                originalFilenames.add(file.getOriginalFilename());
            }
            AjaxResult ajax = success();
            ajax.put("urls", StringUtils.join(urls, FILE_DELIMETER));
            ajax.put("fileNames", StringUtils.join(fileNames, FILE_DELIMETER));
            ajax.put("newFileNames", StringUtils.join(newFileNames, FILE_DELIMETER));
            ajax.put("originalFilenames", StringUtils.join(originalFilenames, FILE_DELIMETER));
            return ajax;
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 本地资源通用下载
     */
    @GetMapping("/download/resource")
    public void resourceDownload(String resource, HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        try {
            if (!FileUtils.checkAllowDownload(resource)) {
                throw new Exception(StringUtils.format("资源文件({})非法，不允许下载。 ", resource));
            }
            // 本地资源路径
            String localPath = RuoYiConfig.getProfile();
            // 数据库资源地址
            String downloadPath = localPath + StringUtils.substringAfter(resource, Constants.RESOURCE_PREFIX);
            // 下载名称
            String downloadName = StringUtils.substringAfterLast(downloadPath, "/");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, downloadName);
            FileUtils.writeBytes(downloadPath, response.getOutputStream());
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    /**
     * 导入execl数据（单个文件）文件名即表名
     */

    @Anonymous
    @ApiOperation("上传文件")
    @PostMapping(value = "/uploadData", headers = "content-type=multipart/form-data")
    public AjaxResult uploadDataFile(String tableName, @RequestPart("file") MultipartFile file) throws Exception {
        Map<String, Object> map = new HashMap<>();
        map.put("tableName", "test");
        map.put("databaseType", "mysql");
        map.put("databaseName", "zkqy-call");
        try {
            // 创建一个工作簿对象 解析上传的excel表数据
            List<Map<String, String>> listMap = convertList(EasyExcel.read(file.getInputStream()).sheet().headRowNumber(0).doReadSync());
            // 查询当前文件插入表结构
            List<TableInfo> tableInfoList = dataSourceService.tableFieldInfo(map);
            Map<String, Object> fieldMap = new HashMap<>(); // 存储 字段描述：字段列名
            tableInfoList.forEach(item -> {
                fieldMap.put(item.getFieldDescription(), item.getFieldName());
            });
            listMap.get(0).forEach((mKey, mVal) -> {
                if (mVal != null) listMap.get(0).put(mKey, fieldMap.get(mVal).toString());
            });
            // 挣个excel文件数据
            listMap.forEach(item -> {
                if (listMap.indexOf(item) == 0) return;
                Map<String, String> handleMap = new HashMap<>(item);
                handleMap.forEach((mKey, mVal) -> {
                    // 新增       // 删除当前map中的k/v
                    item.put(listMap.get(0).get(mKey), item.remove(mKey));
                });
            });
            listMap.remove(0);
            CommonEntity commonEntity = new CommonEntity();
            commonEntity.setBasicMap(JSONObject.parseObject("{ \"tableName\":\"test\"}", new TypeReference<Map<String, Object>>() {
            }));
            commonEntity.setAddListMap(listMap);
            return success(commonService.batchInsert(commonEntity));
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }


    // map类型转换
    public static List<Map<String, String>> convertList(List<Map<Integer, String>> inputList) {
        List<Map<String, String>> resultList = new ArrayList<>();
        for (Map<Integer, String> inputMap : inputList) {
            Map<String, String> resultMap = new HashMap<>();
            for (Map.Entry<Integer, String> entry : inputMap.entrySet()) {
                Integer key = entry.getKey();
                String value = entry.getValue();
                // 将 Integer 类型的 key 转换为 String 类型
                String convertedKey = key.toString();
                resultMap.put(convertedKey, value);
            }
            resultList.add(resultMap);
        }
        return resultList;
    }
}
